package PowerListingFieldEditor::CMS;

use strict;

use MT;
use MT::ListProperty;
use MT::Util;
use MT::Util qw( archive_file_for first_n_words encode_html epoch2ts format_ts relative_date encode_url );
use CustomFields::Field;
use CustomFields::App::CMS;

use PowerListingFieldEditor::CMS::Asset;
use Data::Dumper;

sub init_request {
    my ($eh, $app) = @_;

    my $plugin = $app->component("PowerListingFieldEditor");
    my $field_column = $plugin->get_config_value('field_column', 'system');
    if (!$field_column) {
        $plugin->set_config_value('field_column', time(), 'system');
    }
    $plugin->set_config_value('rest_day', rest_day(), 'system');
    return if is_limit();

    my $q = $app->param;
    my $mode = $q->param('__mode');
    return if !($mode eq 'list' || $mode eq 'filtered_list' || $mode eq 'save_from_list');

    my $type = $q->param('_type') || $app->param('datasource');
#    return if !($type eq 'entry' || $type eq 'page' || $type eq 'comment' || $type eq 'asset' || $type eq 'member');
    return if !($type eq 'entry' || $type eq 'page');

    my $blog_id = $q->param('blog_id') || return;

    my $blog = MT::Blog->load($blog_id);
    return if !$blog;
#    return if $blog->class eq 'website';

    my $asset_type = [qw( file audio image video)] if $type eq 'asset';

    my @data1 = [
        {   class   => 'status2',
            label   => 'Status',
            display => 'default',
        },
        {   class   => 'view-link2',
            label   => 'Link',
            display => 'default',
        },
    ];

    my $plugin = $app->component('PowerListingFieldEditor');
    my $prop;

    my $order = 100;

    $prop->{status3} = {
        label     => 'Status3',
        col       => 'status',
        display   => 'optional',
        html      => \&out_html_default_status,
        col_class => 'icon',
        order     => $order += 10,
        base      => '__virtual.single_select',
        single_select_options => [
              { label => MT->translate('Draft'),     value => 1, },
              { label => MT->translate('Published'), value => 2, },
              { label => MT->translate('Reviewing'), value => 3, },
              { label => MT->translate('Scheduled'), value => 4, },
              { label => MT->translate('Junk'),      value => 5, },
        ],
        filter_editable => 0
    };

    $prop->{"blog${blog_id}_title"} = {
        label => 'title2',
        col => 'title',
        column =>  'title2',
        display => 'default',
        html => \&out_html_default_title,
        bulk_sort => \&bulk_sort_default,
        order => $order += 10,
        sub_fields => @data1,
        filter_tmpl    => '<mt:var name="filter_form_string">',
        terms     => sub {
            my $prop = shift;
            my ( $args, $db_terms, $db_args ) = @_;
            my $col    = $prop->col or die;
            my $option = $args->{option};
            my $query  = $args->{string};
            if ( 'contains' eq $option ) {
                $query = { like => "%$query%" };
            }
            elsif ( 'not_contains' eq $option ) {
                $query = { not_like => "%$query%" };
            }
            elsif ( 'beginning' eq $option ) {
                $query = { like => "$query%" };
            }
            elsif ( 'end' eq $option ) {
                $query = { like => "%$query" };
            }
            if ( $prop->is_meta ) {
                return $prop->join_meta( $db_args, $query );
            }
            else {
                return { $col => $query };
            }
        },

    };

    foreach my $col (qw( text excerpt text_more keywords tags )) {
        my $key = "blog${blog_id}_".$col;
        $prop->{$key} = {
            label => $plugin->translate($col),
            column =>  $col,
            display => $col eq 'authored_on' ? 'default' : 'optional',
            html => \&out_html_default,
            bulk_sort => \&bulk_sort_default,
            type => $col =~ /text|excerpt/ ? 'textarea' :
                    $col =~ /_on/ ? 'date' : 'text',
            order => $order += 10,
        };
    }

    $prop->{"blog${blog_id}_authored_on"} = {
        label      => $plugin->translate('authored_on'),
        column     => 'authored_on',
        display    => 'optional',
        html       => \&out_html_default,
        bulk_sort  => \&bulk_sort_default,
        default_sort_order => 'descend',
        col_class     => 'date',
        type => 'date',
        order      => $order += 10,
    };

    $prop->{"blog${blog_id}_modified_on"} = {
        label      => $plugin->translate('modified_on'),
        column     => 'modified_on',
        display    => 'optional',
        html       => \&out_html_default,
        bulk_sort => \&bulk_sort_default,
        col_class     => 'date',
        type => 'date',
        order      => $order += 10,
    };

    $prop->{"blog${blog_id}_unpublished_on"} = {
        label      => $plugin->translate('unpublished_on'),
        column     => 'unpublished_on',
        display    => 'optional',
        html       => \&out_html_default,
        bulk_sort => \&bulk_sort_default,
        col_class     => 'date',
        type => 'date',
        order      => $order += 10,
    };

    $prop->{status2} = {
        label     => 'Status2',
        col       => 'status',
        display   => 'none',
        col_class => 'icon',
        base      => '__virtual.single_select',
        single_select_options => [
              { label => MT->translate('Draft'),     value => 1, },
              { label => MT->translate('Published'), value => 2, },
              { label => MT->translate('Reviewing'), value => 3, },
              { label => MT->translate('Scheduled'), value => 4, },
              { label => MT->translate('Junk'),      value => 5, },
        ],
        filter_editable => 0
    };

    my @category_list;
    my $class;
    if ( $type eq 'entry' ) {
        $class = MT->model('category');
    } elsif ( $type eq 'page' ) {
        $class = MT->model('folder');
    }
    my @categories = $class->load( { blog_id => $blog_id } );
    for my $category (@categories) {
        push @category_list, { label => $category->label, value => $category->id };
    }

    $prop->{category2} = {
        label     => $type eq 'entry' ? 'Category2' : 'Folder',
        col       => 'category',
        display   => 'optional',
        html      => \&out_html_default_category,
        col_class => 'icon',
        view_filter      => 'blog',
        category_class   => $type eq 'entry' ? 'category' : 'folder',
        order     => $order += 10,
        base      => '__virtual.single_select',
        single_select_options => \@category_list,

        terms => sub {
            my ( $prop, $args, $db_terms, $db_args ) = @_;
            my $blog_id = MT->app->blog->id;
            my $app     = MT->instance;
            my $option  = $args->{option};

            my $category = MT->model( $prop->category_class )->load( $args->{value} );
            $args->{string} = $category->label;

            my $query   = $args->{string};
            push @{ $db_args->{joins} }, MT->model('placement')->join_on(
                undef,
                {   entry_id => \'= entry_id',
                    blog_id  => $blog_id,
                },
                {   unique => 1,
                    join   => MT->model( $prop->category_class )->join_on(
                        undef,
                        {   label   => $query,
                            id      => $type eq 'entry' ? \'= placement_category_id' : \'= placement_folder_id',
                            blog_id => $blog_id,
                        },
                        { unique => 1, }
                    ),
                },
            );
            return;
        },
    };

    my $asset_key = "blog${blog_id}_include_asset_ids";
    $prop->{$asset_key} = {
        label   => 'Asset',
        col     => 'include_asset_ids',
        display => 'optional',
        html    => \&out_html_default_asset,
        type    => 'image',
        order   => $order += 10,
    };
##################

    my @fields = CustomFields::Field->load({
        obj_type => ($asset_type) ? $asset_type : $type eq 'member' ? 'author' : $type,
        blog_id => $blog_id ? [ $blog_id, 0 ] : 0,
    });
#    return if !@fields;

    my @fields_image;
    my $order = 1000;
    foreach my $field (@fields) {
        if (!( $field->type eq 'text' ||
                   $field->type eq 'textarea' ||
                   $field->type eq 'checkbox' ||
                   $field->type eq 'select' ||
                   $field->type eq 'radio' ||
                   $field->type eq 'url' )) {
            push @fields_image, $field;
            next;
        }
        my $key = "blog${blog_id}_".$field->basename;
        $prop->{$key} = {
            label => $field->name,
            display => 'optional',
            html => \&out_html,
            bulk_sort => \&bulk_sort,
            type => $field->type,
            basename => $field->basename,
            order => $order += 100,
            option => $field->options ? ($field->options) : '',
        };
    }
    foreach my $field (@fields_image) {
        next if !( $field->type eq 'image' );
        my $key = "blog${blog_id}_".$field->basename;
        $prop->{$key} = {
            label => $field->name,
            display => 'optional',
            html => \&out_html,
            type => $field->type,
            basename => $field->basename,
            order => $order += 100,
        };
    }
    return 1 if !$prop;

    $plugin->registry({
        list_properties => {
            $type => $prop,
        },
    });

    my $plugin = MT->component("PowerListingFieldEditor");
    my $field_column = $plugin->get_config_value('field_column', 'system');
    if (!$field_column) {
        $plugin->set_config_value('field_column', time(), 'system');
    }

    my $core = MT->component("core");
    $core->registry("applications","cms","content_actions","entry","reset_all_counter",
        {
            class => 'icon-action',
            id    => 'action-edit-all',
            label => $plugin->translate('Edit all'),
            order => 100,
        });

    my $core = MT->component("core");
    $core->registry("list_properties","entry","title",
        { display => 'optional' });
    $core->registry("list_properties","entry","authored_on",
        { 
            auto       => 1,
             display   => 'none',
            label      => 'Publish Date',
            use_future => 1,
            order      => 600,
        });
    $core->registry("list_properties","entry","modified_on",
        { base  => '__virtual.modified_on', display => 'none' });
    $core->registry("list_properties","entry","unpublished_on",
        { 
            auto       => 1,
             display   => 'none',
            label      => 'Unpublish Date',
            use_future => 1,
            order      => 750,
        });
    $core->registry("list_properties","entry","category",
        { display => 'none' });

    $core->registry("list_properties","entry","id",
        { default_sort_order  => 'descend',
          base  => '__virtual.id',
          order => 100,
        });
    $core->registry("list_properties","page","id",
        { default_sort_order  => 'descend',
          base  => '__virtual.id',
          order => 100,
        });

    $core->registry("listing_screens","entry",
        { default_sort_key => "blog${blog_id}_authored_on" });
    $core->registry("listing_screens","page",
        { default_sort_key => "blog${blog_id}_authored_on" });

    1;
}

sub bulk_sort {
    my $prop = shift;
    my ($objs) = @_;
    my $basename = $prop->basename;
    my %entry_id = map { $_->id => 1 } @$objs;
    my @entries = MT->model('entry')->load( { id => [ keys %entry_id ] } );
    my %data = map { $_->id => $_->meta('field.' . $basename) } @entries;
    return sort {
        $data{ $a->id } cmp $data{ $b->id }
    } @$objs;
}

sub out_html_default_status {
    my ( $prop, $obj ) = @_;

    my $meta = $obj->status;
    my $status_label = MT->translate( MT::Entry::status_text( $obj->status ) );
    $meta = '---' if !$meta;
    my $id = $obj->id;

    my $options = $prop->single_select_options;
    my $buf;
    $buf .= qq{<option value=""></option>};
    for my $option (@{$options}) {
        my $value = $option->{value};
        my $label = $option->{label};
        if ( $value eq $meta ) {
            $buf .= qq{<option value="$value" selected="selected">$label</option>};
        } else {
            $buf .= qq{<option value="$value">$label</option>};
        }
    }
    my $element;
    $element = qq{<select name="note-status-$id" id="note-status-$id" class="lfeselect">} . $buf . qq{</select>};

    my $save_changes_label = MT->translate('Save Changes');
    my $save_label = MT->translate('Save');
    my $cancel_label = MT->translate('Cancel');

    my $out;
    $out = qq{<span id="note-status-link-$id" class="view-link"><a href="#$id" class="start-edit" title="}.encode_html($meta).qq{">}.$status_label.qq{</a></span><span id="note-status-field-$id" style="display: none">} . $element . qq{<span class="buttons"><a href="#$id" title="$save_changes_label" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
    return $out;
}

sub out_html_default_category {
    my ( $prop, $obj ) = @_;

    my $id = $obj->id;
    my ( $args, $terms, $join_terms, $join_args );
    $join_terms->{ entry_id } = $id;
    $join_terms->{ is_primary } = 1;
    $args->{ join } = MT::Placement->join_on( 'category_id', $join_terms );
    $terms->{ blog_id } = MT->app->blog->id;

    my @categories = MT->model( $prop->category_class )->load( $terms, $args );
    my $meta;
    my $label;
    for my $cat (@categories) {
        $label = $cat->label;
        $meta = $cat->id;
    }
    $label = '---' if !$meta;

    my $options = $prop->single_select_options;
    my $buf;
    $buf .= qq{<option value=""></option>} if !$meta;
    for my $option (@{$options}) {
        my $value = $option->{value};
        my $label = $option->{label};
        if ( $value eq $meta ) {
            $buf .= qq{<option value="$value" selected="selected">$label</option>};
        } else {
            $buf .= qq{<option value="$value">$label</option>};
        }
    }
    my $element;
    $element = qq{<select name="note-category-$id" id="note-category-$id" class="lfeselect">} . $buf . qq{</select>};

    my $save_changes_label = MT->translate('Save Changes');
    my $save_label = MT->translate('Save');
    my $cancel_label = MT->translate('Cancel');

    my $out;
    $out = qq{<span id="note-category-link-$id" class="view-link"><a href="#$id" class="start-edit" title="}.encode_html($meta).qq{">}.$label.qq{</a></span><span id="note-category-field-$id" style="display: none">} . $element . qq{<span class="buttons"><a href="#$id" title="$save_changes_label" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
    return $out;
}

sub out_html_default_asset {
    my ( $prop, $obj ) = @_;

    my $select = MT->translate('Select');
    my $delete = MT->translate('Delete');

    my $id = $obj->id;
    my @object_assets = MT::ObjectAsset->load({ object_ds => 'entry', object_id => $id } );

    my $save_changes_label = MT->translate('Save Changes');
    my $save_label = MT->translate('Save');
    my $cancel_label = MT->translate('Cancel');
    my $element;

    my $meta = '';

    my $asset_id = '';
    my @asset_ids;
    for my $object_asset (@object_assets) {
        my @assets = MT->model('asset')->load( { id => $object_asset->asset_id } );
        for (@assets) {
            my ( $url, $w, $h ) = $_->thumbnail_url( Height => 45, Width => 45, Square => 1 );
            $meta .= '<img id="image-' . 'include_asset_ids' . '-' . $object_asset->asset_id . '-child" src="'.$url.'" />';
            $asset_id = $_->id;
            push @asset_ids, $_->id;
        }
    }
    $meta = '---' if !@object_assets;

    my $uri = MT->instance->app_uri;
    my $blog_id = MT->instance->blog->id;
    $element = qq{<div id="image-include_asset_ids-$id">$meta</div><div class="image-link"><a class="mt-open-dialog" href="${uri}?__mode=list_asset_for_list&_type=asset&blog_id=$blog_id&obj_id=$id&basename=include_asset_ids&dialog_view=1&filter=class&filter_val=image&require_type=image&edit_field=customfield_cfimage&asset_select=1'">} . $select . qq{</a> <a href="#$id" class="clear-image" id="delete-include_asset_ids-$id">} . $delete . qq{</a></div>};

    my $out;
    $out = qq{<span id="note-include_asset_ids-link-$id" class="view-link"><a href="#$id" class="start-edit" title="">}. $meta .qq{</a></span><span id="note-include_asset_ids-field-$id" style="display: none">} . $element . qq{<input class="include_asset_ids-$id" id="include_asset_ids-include_asset_ids-$id" type="hidden" value="$asset_id" name="include_asset_ids" /><input id="note-include_asset_ids-$id" class="asset_ids-$id" type="hidden" value="} . join(',',@asset_ids) . qq{" name="include_asset_ids" /><span class="buttons"><a href="#$id" title="$save_changes_label" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
    return $out;
}

sub out_html_default_title {
    my ( $prop, $obj ) = @_;

    my $class       = $obj->class;
    my $class_label = $obj->class_label;
#    my $title       = $prop->super(@_);

    my $value = $obj->title;
    $value = '---' if !$value;
    my $id = $obj->id;
    my $save_changes_label = MT->translate('Save Changes');
    my $save_label = MT->translate('Save');
    my $cancel_label = MT->translate('Cancel');
    my $element = qq{<input type="text" name="note-title-$id" id="note-title-$id" class="lfetext" value="} . encode_html($value) . qq{" />};
    my $title;
    $title = qq{<span id="note-title-link-$id" class="view-link"><a href="#$id" class="start-edit" title="}.encode_html($value).qq{">}.encode_html($value).qq{</a></span><span id="note-title-field-$id" style="display: none">} . $element . qq{<span class="buttons"><a href="#$id" title="$save_changes_label" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};


#    my $id        = $obj->id;
    my $permalink = MT::Util::encode_html( $obj->permalink );
    my $edit_url  = MT->app->uri(
        mode => 'view',
        args => {
            _type   => $class,
            id      => $obj->id,
            blog_id => $obj->blog_id,
        }
    );
    my $status = $obj->status;
    my $status_class
        = $status == MT::Entry::HOLD()    ? 'Draft'
        : $status == MT::Entry::RELEASE() ? 'Published'
        : $status == MT::Entry::REVIEW()  ? 'Review'
        : $status == MT::Entry::FUTURE()  ? 'Future'
        : $status == MT::Entry::JUNK()    ? 'Junk'
        :                                   '';
    my $lc_status_class = lc $status_class;
    require MT::Entry;
    my $status_file
        = $status == MT::Entry::HOLD()    ? 'draft.gif'
        : $status == MT::Entry::RELEASE() ? 'success.gif'
        : $status == MT::Entry::REVIEW()  ? 'warning.gif'
        : $status == MT::Entry::FUTURE()  ? 'future.gif'
        : $status == MT::Entry::JUNK()    ? 'warning.gif'
        :                                   '';
    my $status_img
        = MT->static_path . 'images/status_icons/' . $status_file;
    my $view_img
        = MT->static_path . 'images/status_icons/view.gif';
    my $view_link = $obj->status == MT::Entry::RELEASE()
        ? qq{
        <span class="view-link2">
          <a href="$permalink" target="_blank">
            <img alt="View $class_label" src="$view_img" />
          </a>
        </span>
    }
        : '';

    my $out = qq{
        <span class="icon status2 $lc_status_class">
          <a href="$edit_url"><img alt="$status_class" src="$status_img" /></a>
        </span>
        <span class="title">
          $title
        </span>
        $view_link
    };
    return $out;
}

sub out_html_default {
    my ( $prop, $obj ) = @_;

    my $col = $prop->column eq 'title2' ? 'title' : $prop->column;
#    my $col = $prop->column;

my $value;
my $form_value;
if ($col eq 'tags') {
    my $tag_delim = chr( MT->app->user->entry_prefs->{tag_delim} );
    require MT::Tag;
    $form_value = $value = MT::Tag->join( $tag_delim, $obj->tags );
} else {
    $form_value = $value = $obj->$col;
}
#    my $value = $obj->$col;
#    my $form_value = $obj->$col;
    if ($prop->type ne 'checkbox') {
        $form_value = ''  if !$value;
        $value = '---' if !$value;
    } else {
        $value = '0' if !$value;
    }
    my $id = $obj->id;

    my $save_changes_label = MT->translate('Save Changes');
    my $save_label = MT->translate('Save');
    my $cancel_label = MT->translate('Cancel');
    my $element;
    my $label;
    if ($prop->type eq 'text') {
        $label = $value;
        $element = qq{<input type="text" name="note-$col-$id" id="note-$col-$id" class="lfetext" value="} . encode_html($form_value) . qq{" />};
    } elsif ($prop->type eq 'date') {
        my $plugin = MT->component('PowerListingFieldEditor');
        my $y = $plugin->translate('year');
        my $m = $plugin->translate('month');
        my $d = $plugin->translate('day');
        my $h = $plugin->translate('hour');
        my $min = $plugin->translate('minuite');
        my $s = $plugin->translate('second');
        ($label = $value) =~ s/(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})/$1$y$2$m$3$d/;
        $value =~ s/(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})/$1-$2-$3 $4:$5:$6/;
        $element = qq{<input type="text" name="note-$col-$id" id="note-$col-$id" class="lfedate" value="} . encode_html($value) . qq{" />};
    } elsif ($prop->type eq 'textarea') {
        $label = $value;
        $element = qq{<textarea name="note-$col-$id" id="note-$col-$id" class="lfetextarea">} . encode_html($form_value) . qq{</textarea>};
    }
    my $out;
    $out = qq{<span id="note-$col-link-$id" class="view-link"><a href="#$id" class="start-edit" title="}.encode_html($value).qq{">}.encode_html($label).qq{</a></span><span id="note-$col-field-$id" style="display: none">} . $element . qq{<span class="buttons"><a href="#$id" title="$save_changes_label" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
    return $out;
}

sub bulk_sort_default {
    my $prop = shift;
    my ($objs) = @_;
    my $col = $prop->column eq 'title2' ? 'title' : $prop->column;
#    my $col = $prop->column;
    my %entry_id = map { $_->id => 1 } @$objs;
    my @entries = MT->model('entry')->load( { id => [ keys %entry_id ] } );
    my %data = map { $_->id => $_->$col } @entries;
    return sort {
        $data{ $a->id } cmp $data{ $b->id }
    } @$objs;
}

sub out_html {
    my ( $prop, $obj ) = @_;

    my $select = MT->translate('Select');
    my $delete = MT->translate('Delete');

    my $basename = $prop->basename;
    my $meta = $obj->meta('field.' . $basename);
    if ($prop->type ne 'checkbox') {
        $meta = '---' if !$meta;
    } else {
        $meta = '0' if !$meta;
    }
    my $id = $obj->id;

    my $asset_id;
    my $orig_meta;

    my $save_changes_label = MT->translate('Save Changes');
    my $save_label = MT->translate('Save');
    my $cancel_label = MT->translate('Cancel');
    my $element;
    if ($prop->type eq 'text') {
        $element = qq{<input type="text" name="note-$basename-$id" id="note-$basename-$id" class="lfetext" value="} . encode_html($meta) . qq{" />};
    } elsif ($prop->type eq 'textarea') {
        $element = qq{<textarea name="note-$basename-$id" id="note-$basename-$id" class="lfetextarea">} . encode_html($meta) . qq{</textarea>};
    } elsif ($prop->type eq 'url') {
        $element = qq{<input type="text" name="note-$basename-$id" id="note-$basename-$id" class="lfeurl" value="} . encode_html($meta) . qq{" />};
    } elsif ($prop->type eq 'checkbox') {
        my $flag = $meta eq '0' ? '' : 'checked="checked"';
        $element = qq{<div><input type="checkbox" name="note-$basename-$id" id="note-$basename-$id" class="lfecheckbox" } . $flag . qq{ /></div>};
    } elsif ($prop->type eq 'select') {
        my @options = split(',', $prop->option);
        my $buf;
        $buf .= qq{<option value=""></option>};
        for my $option (@options) {
            if ( $option eq $meta ) {
                $buf .= qq{<option value="$option" selected="selected">$option</option>};
            } else {
                $buf .= qq{<option value="$option">$option</option>};
            }
        }
        $element = qq{<select name="note-$basename-$id" id="note-$basename-$id" class="lfeselect">} . $buf . qq{</select>};
    } elsif ($prop->type eq 'radio') {
        my @options = split(',', $prop->option);
        for my $option (@options) {
            if ( $option eq $meta ) {
                $element .= qq{<input type="radio" name="note-$basename-$id" value="$option" checked="checked" /> } . encode_html($option) . '<br />';
            } else {
                $element .= qq{<input type="radio" name="note-$basename-$id" value="$option" /> } . encode_html($option) . '<br />';
            }
        }
        $element = qq{<div id="note-$basename-$id">} . $element . qq{</div>};
    } elsif ($prop->type eq 'image') {
        $orig_meta = $meta;
        $meta =~ s/.*asset-id="(\d+)".*/$1/;
        my @assets = MT->model('asset')->load( { id => $meta } );
        for (@assets) {
            my ( $url, $w, $h ) = $_->thumbnail_url( Height => 45, Width => 45, Square => 1 );
            $meta = '<img id="image-' . $basename . '-' . $id . '-child" src="'.$url.'" />';
$asset_id = $_->id;
        }
        my $uri = MT->instance->app_uri;
        my $blog_id = MT->instance->blog->id;
        $element = qq{<div id="image-$basename-$id">$meta</div><div class="image-link"><a class="mt-open-dialog" href="${uri}?__mode=list_asset_for_list&_type=asset&blog_id=$blog_id&obj_id=$id&basename=$basename&dialog_view=1&filter=class&filter_val=image&require_type=image&edit_field=customfield_cfimage&asset_select=1'">} . $select . qq{</a> <a href="#$id" class="clear-image" id="delete-$basename-$id">} . $delete . qq{</a></div>};
    }
    my $out;
    if ($prop->type ne 'image') {
        $out = qq{<span id="note-$basename-link-$id" class="lfe-view-link"><a href="#$id" class="start-edit" title="}.encode_html($meta).qq{">}.encode_html($meta).qq{</a></span><span id="note-$basename-field-$id" style="display: none">} . $element . qq{<span class="buttons"><a href="#$id" title="$save_changes_label" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
    } else {
        $out = qq{<span id="note-$basename-link-$id" class="lfe-view-link"><a href="#$id" class="start-edit" title="">}. $meta .qq{</a></span><span id="note-$basename-field-$id" style="display: none">} . $element . qq{<input class="include_asset_ids-$id" id="include_asset_ids-$basename-$id" type="hidden" value="$asset_id" name="include_asset_ids" /><input id="note-$basename-$id" class="customfield_$basename-$id" type="hidden" value="} . encode_html($orig_meta) . qq{" name="customfield_$basename" /><span class="buttons"><a href="#$id" title="$save_changes_label" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
    }
    return $out;
}

sub add_script {
    my ($cb, $app, $param, $tmpl) = @_;
    my $type = $app->param('_type');
    return if is_limit();

    my $asset_type = [qw( file audio image video)] if $type eq 'asset';

    my @fields = CustomFields::Field->load({
        obj_type => ($asset_type) ? $asset_type : $type eq 'member' ? 'author' : $type,
        blog_id => $app->blog ? [ $app->blog->id, 0 ] : 0,
    });
#    return if !@fields;

#    $type = 'member' if $type eq 'author';

    my $old_buf;
    my $link_buf;
    my $field_buf;
    my $note_buf;
    my $old_note_buf;
    my $empty_note_buf;
    my $empty_old_note_buf;
    my $recover_buf;
    my $clear_note_buf;
    my $new_note_buf;
    my @cancel_buf;
    my $submit_buf;
    my $render_buf;

    my $all_note_buf;
    my $all_old_note_buf;
    my $all_empty_note_buf;
    my $all_empty_old_note_buf;
    my $all_recover_buf;
    my $all_clear_note_buf;
    my $all_new_note_buf;
    my @all_cancel_buf;
    my $all_submit_buf;
    my $all_render_buf;

    my @basenames;
    my $counter = 0;

    my $image_buf;
    my $old_image_buf;
    my $recover_image_buf;
    my $clear_image_buf;
    my $asset_ids;

    my $all_image_buf;
    my $all_old_image_buf;
    my $all_recover_image_buf;
    my $all_clear_image_buf;
    my $all_asset_ids;

    my $all_note_buf_new;
    my $all_old_note_buf_new;
    my $all_check_note_buf_new;

    foreach my $field (@fields) {
        next if !( $field->type eq 'text'     ||
                   $field->type eq 'textarea' ||
                   $field->type eq 'checkbox' ||
                   $field->type eq 'select'   ||
                   $field->type eq 'radio'    ||
                   $field->type eq 'url'      ||
                   $field->type eq 'image' );
            $all_note_buf_new .= "all_note_" . $field->basename . " = new Array();\n";
            $all_old_note_buf_new .= "all_old_note_" . $field->basename . " = new Array();\n";
            $all_check_note_buf_new .= "all_check_note_" . $field->basename . " = new Array();\n";
    }

    foreach my $col (qw( title text excerpt text_more keywords include_asset_ids status authored_on modified_on unpublished_on category tags )) {
        $old_buf .=  "var old_note_" . $col . " = '';\nvar check_note_" . $col . " = '';\n";
        $link_buf .=  "note_links.push( jQuery('#note-" . $col . "-link-' + id) );\n";
        $field_buf .= "note_fields.push( jQuery('#note-" . $col . "-field-' + id) );\n";

        # note
        $note_buf .= "var note_" . $col . " = jQuery('#note-" . $col . "-' + id);\n";

        # old
        if ($col eq 'allow_comments' || $col eq 'allow_pings') {
            $old_note_buf .= "old_note_" . $col . " =  jQuery('#note-" . $col . "-' + id).attr('checked');";
        } else {
            $old_note_buf .= "old_note_" . $col . " = note_" . $col . ".val();\ncheck_note_" . $col . " = note_" . $col . ".val();\n";
        }

        # empty
        if (!($col eq 'allow_comments' || $col eq 'allow_pings')) {
            $empty_note_buf .= "if (note_" . $col . ".val() == '---'){\n  note_" . $col . ".val('');\n  check_note_" . $col . " = '';\n}\n";
        } else {
            $empty_note_buf .= "if (jQuery('#note-" . $col . "-' + id).attr('checked') == true) { note_" . $col . "= 1;\ncheck_note_" . $col . "= 1; } else { note_" . $col . "= 0;\ncheck_note_" . $col . "= 0; }";
        }

        # recover
        if (!($col eq 'allow_comments' || $col eq 'allow_pings')) {
            $recover_buf .= "note_" . $col . ".val( old_note_" . $col . " );\n";
        } else {
            $recover_buf .= "note_" . $col . ".attr('checked', old_note_" . $col . ");\n";
        }
        $clear_note_buf .=  "old_note_" . $col . " = '';\n";

        # new
        if ($col eq 'allow_comments' || $col eq 'allow_pings') {
            $new_note_buf .= "var new_note_" . $col . ";\nif (jQuery('#note-" . $col . "-' + id).attr('checked') == true) { new_note_" . $col . "= 1; } else { new_note_" . $col . "= 0; }";
        } else {
            $new_note_buf .= "var new_note_" . $col . " = note_" . $col . ".val();\n";
        }

        push @cancel_buf, "check_note_" . $col . " == new_note_" . $col;

        # submit
        if ($col eq 'allow_comments' || $col eq 'allow_pings') {
            $submit_buf .= "var note_" . $col . ";\nif (jQuery('#note-" . $col . "-' + id).attr('checked') == true) { note_" . $col . "= 1; } else { note_" . $col . "= 0; }";
            $all_submit_buf .= "var note_" . $col . ";\nif (jQuery('#note-" . $col . "-' + id).attr('checked') == true) { note_" . $col . "= 1; } else { note_" . $col . "= 0; }";
        } else {
            $submit_buf .= "var note_" . $col . " = jQuery('#note-" . $col . "-' + id).val();\n";
            $all_submit_buf .= "var note_" . $col . " = jQuery('#note-" . $col . "-' + id).val();\n";
        }

        # render
        if ($col eq 'allow_comments' || $col eq 'allow_pings') {
            $render_buf .= $col . "_cb_beacon: 1,\n";
#            $all_render_buf .= "param.append('" . $col . "_cb_beacon', 1);\n";
#            $all_render_buf .= $col . "_cb_beacon: 1,\n";
            $all_render_buf .= "param['" . $col . "_cb_beacon'] = 1;\n";
        }
        $render_buf .= $col . ": note_" . $col . ",\n";
#        $all_render_buf .= "param.append('" . $col . "', note_" . $col . ");\n";
#        $all_render_buf .= $col . ": note_" . $col . ",\n";
        $all_render_buf .= "param['" . $col . "'] =  note_" . $col . ";\n";

        $counter++;
    }

    $old_image_buf .= "var old_image_" . 'include_asset_ids' . " = '';\n";
    $image_buf .= "old_image_" . 'include_asset_ids' . " = jQuery('#image-" . 'include_asset_ids' . "-' + id).html();\n";
    $recover_image_buf .= "if(old_note_" . 'include_asset_ids' . " == '---') {\nold_note_" . 'include_asset_ids' . " = '';\n}\njQuery('#image-" . 'include_asset_ids' . "-' + id).html( old_image_" . 'include_asset_ids' . " );\nif (old_image_" . 'include_asset_ids' . " == '---'){\n  jQuery('#delete-" . 'include_asset_ids' . "-' + id).css('display', 'none');\n} else {\n  jQuery('#delete-" . 'include_asset_ids' . "-' + id).css('display', 'inline');\n}\n";
    $clear_image_buf .=  "old_image_" . 'include_asset_ids' . " = '';\n";

    foreach my $field (@fields) {
        next if !( $field->type eq 'text'     ||
                   $field->type eq 'textarea' ||
                   $field->type eq 'checkbox' ||
                   $field->type eq 'select'   ||
                   $field->type eq 'radio'    ||
                   $field->type eq 'url'      ||
                   $field->type eq 'image' );
        $old_buf .=  "var old_note_" . $field->basename . " = '';\nvar check_note_" . $field->basename . " = '';\n";
        $link_buf .=  "note_links.push( jQuery('#note-" . $field->basename . "-link-' + id) );\n";
        $field_buf .= "note_fields.push( jQuery('#note-" . $field->basename . "-field-' + id) );\n";

        # note
        if ($field->type eq 'radio') {
            $note_buf .= "var note_" . $field->basename . " = jQuery('[name=\"note-" . $field->basename . "-' + id + '\"]:checked');\n";
            $all_note_buf .= "all_note_" . $field->basename . "[id] = jQuery('[name=\"note-" . $field->basename . "-' + id + '\"]:checked');\n";
        } else {
            $note_buf .= "var note_" . $field->basename . " = jQuery('#note-" . $field->basename . "-' + id);\n";
            $all_note_buf .= "all_note_" . $field->basename . "[id] = jQuery('#note-" . $field->basename . "-' + id);\n";
        }

        # old
        if ($field->type eq 'checkbox') {
            $old_note_buf .= "old_note_" . $field->basename . " =  jQuery('#note-" . $field->basename . "-' + id).attr('checked');";
            $all_old_note_buf .= "all_old_note_" . $field->basename . "[id] =  jQuery('#note-" . $field->basename . "-' + id).attr('checked');";
        } else {
            $old_note_buf .= "old_note_" . $field->basename . " = note_" . $field->basename . ".val();\ncheck_note_" . $field->basename . " = note_" . $field->basename . ".val();\n";
            $all_old_note_buf .= "all_old_note_" . $field->basename . "[id] = all_note_" . $field->basename . "[id].val();\nall_check_note_" . $field->basename . "[id] = all_note_" . $field->basename . "[id].val();\n";
        }

        # empty
        if ($field->type ne 'checkbox') {
            $empty_note_buf .= "if (note_" . $field->basename . ".val() == '---'){\n  note_" . $field->basename . ".val('');\n  check_note_" . $field->basename . " = '';\n}\n";
            $all_empty_note_buf .= "if (all_note_" . $field->basename . "[id].val() == '---'){\n  all_note_" . $field->basename . "[id].val('');\n  all_check_note_" . $field->basename . "[id] = '';\n}\n";
        } else {
#            $empty_note_buf .= "if (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { note_" . $field->basename . "= 1;\ncheck_note_" . $field->basename . "= 1; } else { note_" . $field->basename . "= 0;\ncheck_note_" . $field->basename . "= 0; }";
            $empty_note_buf .= "if (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { note_" . $field->basename . "= 1;\ncheck_note_" . $field->basename . "= 1; } else { note_" . $field->basename . "= 0;\ncheck_note_" . $field->basename . "= 0; }";
#            $all_empty_note_buf .= "if (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { all_note_" . $field->basename . "[id] = 1;\nall_check_note_" . $field->basename . "[id] = 1; } else { all_note_" . $field->basename . "[id] = 0;\nall_check_note_" . $field->basename . "[id] = 0; }";
            $all_empty_note_buf .= "if (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { all_note_" . $field->basename . "[id] = 1;\nall_check_note_" . $field->basename . "[id] = 1; } else { all_note_" . $field->basename . "[id] = 0;\nall_check_note_" . $field->basename . "[id] = 0; }";
        }

        # recover
        if ($field->type ne 'checkbox') {
            $recover_buf .= "note_" . $field->basename . ".val( old_note_" . $field->basename . " );\n//alert(note_" . $field->basename . ".val());";
            $all_recover_buf .= "all_note_" . $field->basename . "[id].val( all_old_note_" . $field->basename . "[id] );\n//alert(all_note_" . $field->basename . "[id].val());";
        } else {
            $recover_buf .= "note_" . $field->basename . ".attr('checked', old_note_" . $field->basename . ");\n";
            $all_recover_buf .= "all_note_" . $field->basename . "[id].attr('checked', all_old_note_" . $field->basename . "[id] );\n";
        }
        $clear_note_buf .=  "old_note_" . $field->basename . " = '';\n";
        $all_clear_note_buf .=  "all_old_note_" . $field->basename . "[id] = '';\n";

        # new
        if ($field->type eq 'checkbox') {
#            $new_note_buf .= "var new_note_" . $field->basename . ";\nif (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { new_note_" . $field->basename . "= 1; } else { new_note_" . $field->basename . "= 0; }";
            $new_note_buf .= "var new_note_" . $field->basename . ";\nif (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { new_note_" . $field->basename . "= 1; } else { new_note_" . $field->basename . "= 0; }";
#            $all_new_note_buf .= "all_new_note_" . $field->basename . "[id];\nif (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { all_new_note_" . $field->basename . "[id] = 1; } else { all_new_note_" . $field->basename . "[id] = 0; }";
            $all_new_note_buf .= "all_new_note_" . $field->basename . "[id];\nif (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { all_new_note_" . $field->basename . "[id] = 1; } else { all_new_note_" . $field->basename . "[id] = 0; }";
        } else {
            $new_note_buf .= "var new_note_" . $field->basename . " = note_" . $field->basename . ".val();\n";
            $all_new_note_buf .= "all_new_note_" . $field->basename . "[id] = all_note_" . $field->basename . "[id].val();\n";
        }

        push @cancel_buf, "check_note_" . $field->basename . " == new_note_" . $field->basename;
        push @all_cancel_buf, "all_check_note_" . $field->basename . "[id] == all_new_note_" . $field->basename . "[id]";

        # submit
        if ($field->type eq 'radio') {
            $submit_buf .= "var note_" . $field->basename . " = jQuery('[name=\"note-" . $field->basename . "-' + id + '\"]:checked').val();\n";
            $all_submit_buf .= "var note_" . $field->basename . " = jQuery('[name=\"note-" . $field->basename . "-' + id + '\"]:checked').val();";
        } elsif ($field->type eq 'checkbox') {
#            $submit_buf .= "var note_" . $field->basename . ";\nif (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { note_" . $field->basename . "= 1; } else { note_" . $field->basename . "= 0; }";
            $submit_buf .= "var note_" . $field->basename . ";\nif (jQuery('#note-" . $field->basename . "-' + id)[0]) { if ( jQuery('#note-" . $field->basename . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) {note_" . $field->basename . "= 1; } else { note_" . $field->basename . "= 0; } } else { note_" . $field->basename . "= undefined; }";
#            $all_submit_buf .= "var note_" . $field->basename . ";if (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { all_note_" . $field->basename . " = 1; } else { all_note_" . $field->basename . " = 0; }";
            $all_submit_buf .= "var note_" . $field->basename . ";\nif (jQuery('#note-" . $field->basename . "-' + id)[0]) { if ( jQuery('#note-" . $field->basename . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) {note_" . $field->basename . "= 1; } else { note_" . $field->basename . "= 0; } } else { note_" . $field->basename . "= undefined; }";
        } else {
            $submit_buf .= "var note_" . $field->basename . " = jQuery('#note-" . $field->basename . "-' + id).val();\n";
            $all_submit_buf .= "var note_" . $field->basename . " = jQuery('#note-" . $field->basename . "-' + id).val();\n";
        }

        # render
        if ($field->type eq 'checkbox') {
#            $render_buf .= "customfield_" . $field->basename . "_cb_beacon: 1,\n";
#            $all_render_buf .= "customfield_" . $field->basename . "_cb_beacon: 1,";
#            $all_render_buf .= "param['customfield_" . $field->basename . "_cb_beacon'] =  1;";
        }
        $render_buf .= "customfield_" . $field->basename . ": note_" . $field->basename . ",\n";
#        $all_render_buf .= "param.append('customfield_" . $field->basename . "', note_" . $field->basename . ");\n";
#        $all_render_buf .= "customfield_" . $field->basename . ": note_" . $field->basename . ",\n";
        $all_render_buf .= "param['customfield_" . $field->basename . "'] =  note_" . $field->basename . ";\n";

        # asset_ids
        $asset_ids = "var asset_ids = new Array();\nvar asset_ids_val = '';\njQuery('.include_asset_ids-' + id).each(function(){\nasset_ids[asset_ids.length] = jQuery(this).val();\n});\n
if(asset_ids.length > 1){\nasset_ids_val = '[' + asset_ids.join(',') + ']';\n} else if(asset_ids.length == 1){\nasset_ids_val = asset_ids[0];\n} else {\nasset_ids_val = 0;\n}\n";

        if ($field->type eq 'image') {
            $old_image_buf .= "var old_image_" . $field->basename . " = '';\n";
            $image_buf .= "old_image_" . $field->basename . " = jQuery('#image-" . $field->basename . "-' + id).html();\n";
            $recover_image_buf .= "if(old_note_" . $field->basename . " == '---') {\nold_note_" . $field->basename . " = '';\n}\njQuery('#image-" . $field->basename . "-' + id).html( old_image_" . $field->basename . " );\nif (old_image_" . $field->basename . " == '---'){\n  jQuery('#delete-" . $field->basename . "-' + id).css('display', 'none');\n} else {\n  jQuery('#delete-" . $field->basename . "-' + id).css('display', 'inline');\n}\n";
            $clear_image_buf .=  "old_image_" . $field->basename . " = '';\n";
        }

        $counter++;
    }
    return 1 if !$counter;

### copy for category version
    if (!$asset_ids) {
        $asset_ids = "asset_ids_val = ''";
    }
###

    my $cancel = join(' && ', @cancel_buf);

    my $plugin = $app->component('PowerListingFieldEditor');
    my $field = $tmpl->getElementById('container');
    my $node = $tmpl->createElement('setvarblock', { name => 'jq_js_include', component => $plugin });

    my $entry_id_message = $plugin->translate('entry_id');
    my $update_message = $plugin->translate('update...');
    my $end_message = $plugin->translate('done update');

    my $html = <<HTML;
    var current_edit = 0;
    $old_buf
    $old_image_buf

    function clear_image(id, basename_id) {
        basename_id.match(/delete-(.*)/);
        var basename_id = RegExp.\$1;
        jQuery('#image-' + basename_id).html('---');
        jQuery('#note-' + basename_id).val('');
        jQuery('#include_asset_ids-' + basename_id).val('');
        jQuery('#delete-' + basename_id).css('display', 'none');
    }

    function edit_note(id) {
        if (current_edit) return;
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        if (!note_links.length || !note_fields.length) return;
        $note_buf
        current_edit = id;
        $old_note_buf
        $image_buf
        $empty_note_buf

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].hide();
            note_fields[i].show();
        }
        return false;
    }

    function cancel_edit(id) {
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        $note_buf
        if (!note_links.length || !note_fields.length) return;
        $recover_image_buf
        $recover_buf
        $clear_image_buf
        $clear_note_buf
        current_edit = 0;

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].show();
            note_fields[i].hide();
        }
        return false;
    }

    function submit_edit(id) {
        if ((0 != current_edit) && (id != current_edit))
            id = current_edit;
        $note_buf
        $new_note_buf

        if ($cancel) {
            cancel_edit(id);
            return false;
        }
        return submit_edit_form(id);
    }

    function submit_edit_form(id) {
        $submit_buf
        $asset_ids
        renderList({
            __mode: 'save_from_list',
            _type: '$type',
            customfield_beacon: 1,
            $render_buf
            include_asset_ids: asset_ids_val,
            id: id,
            forward_list: 1,
            magic_token: 1
        });
        cancelCreateMode();
        return false;
    }

    function setCreateMode () {
        jQuery('#create-inline-notification').show();
        jQuery('#text').val('').focus();
        jQuery('#content-actions').hide();
    }

    function cancelCreateMode () {
        jQuery('#create-inline-notification').hide();
        jQuery('#create-inline-notification input').mtUnvalidate();
//        jQuery('#content-actions').show();

        current_edit = 0;
    }

    jQuery(document).on('click', '.start-edit', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        cancel_edit(current_edit);
        edit_note(id);
        return false;
    });

    jQuery(document).on('click', '.cancel-edit', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        cancel_edit(id);
        return false;
    });

    jQuery(document).on('click', '.submit-note', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        submit_edit(id);
        return false;
    });

    jQuery(window).bind('listReady', function() {
        jQuery('a.mt-open-dialog').mtDialog();

        jQuery('.clear-image').each(function () {
            jQuery(this).attr('href').match(/#(\\d+)/);
            var id = RegExp.\$1;
            jQuery(this).attr('id').match(/delete-(.*)/);
            var basename_id = RegExp.\$1;
            if (jQuery('#image-' + basename_id).html() == '---'){
                jQuery(this).css('display', 'none');
            } else {
                jQuery(this).css('display', 'inline');
            }
        });
    });

    jQuery(document).on('click', '.clear-image', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        var basename = jQuery(this).attr('id');
        clear_image(id, basename);
        return false;
    });

    jQuery('#uploadStatePnl').hide();
    jQuery(window).bind('listReady', function() {

// remove checkbox
jQuery('.col.head.cb.first-visible-child.first-child input').removeAttr('checked');
jQuery('#entry-table tr').removeClass('selected');
jQuery('#page-table tr').removeClass('selected');

        // end save
        if (done) {
            incr();
            updateStatus(last_id + 1);
            done = 0;
            jQuery('#uploadStatePnl').animate({
                opacity: 'toggle',}, { duration: 2000, easing: 'swing',
                complete: function () {
                    jQuery("#content-actions").show();
                    jQuery("#display-options").show();

        jQuery("#menu").show();
        jQuery(".has-menu-nav #content").css('margin-left', '175px');
        jQuery("#sidebar_none a").show();
        jQuery("#sidebar_block a").hide();

                }
            });
        // 一括編集中にフィールドの増減
        } else {
            if(jQuery('#create-inline-notification').css('display') == 'none') {
                jQuery("#content-actions").show();
            }
            if(jQuery('#create-inline-notification').css('display') == 'block') {
                setEditAllMode();
            }
        }
    });

    jQuery("#progressbar").progressbar();
    jQuery('#submit-all-field').click( function() {
        submit_all_edit();
        return false;
    });

    jQuery('#action-edit-all').click( function() {
        jQuery("#content-actions").hide();
//        jQuery("#display-options").hide();
        setEditAllMode();
        return false;
    });

    jQuery('#create-inline-notification button.mt-cancel-action').click(function() {
//        jQuery("#display-options").show();
        cancelEditAllMode();
        return false;
    });

    jQuery('#sidebar_none a').click(function(){
        jQuery("#menu").hide();
        jQuery(".has-menu-nav #content").css('margin-left', '0');

        jQuery("#sidebar_none a").hide();
        jQuery("#sidebar_block a").show();
    });
    jQuery('#sidebar_block a').click(function(){
        jQuery("#menu").show();
        jQuery(".has-menu-nav #content").css('margin-left', '175px');

        jQuery("#sidebar_none a").show();
        jQuery("#sidebar_block a").hide();
    });

    function cancelEditAllMode () {
        jQuery('#entry-table tr').each(function() {
            cancel_edit_all(this.id);

            jQuery('.buttons', jQuery(this)).each(function() {
                jQuery(this).show();
            });
        });
        jQuery('#create-inline-notification').hide();
        jQuery('#content-actions').show();

        jQuery("#menu").show();
        jQuery(".has-menu-nav #content").css('margin-left', '175px');
        jQuery("#sidebar_none a").show();
        jQuery("#sidebar_block a").hide();

        current_edit = 0;
    }

    function setEditAllMode () {
        jQuery('#create-inline-notification').show();
        jQuery('#content-actions').hide();

        cancel_edit(current_edit);

        $all_note_buf_new
        $all_old_note_buf_new
        $all_check_note_buf_new

        jQuery('#entry-table tr').each(function() {
            edit_all_note(this.id);

            jQuery('.buttons', jQuery(this)).each(function() {
                jQuery(this).hide();
            });
        });
        current_edit = 'all';
    }

    function edit_all_note(id) {
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        if (!note_links.length || !note_fields.length) return;
        $all_note_buf
        $all_old_note_buf
        $all_image_buf
        $all_empty_note_buf

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].hide();
            note_fields[i].show();
        }
        return false;
    }

    function cancel_edit_all(id) {
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        $all_note_buf
        if (!note_links.length || !note_fields.length) return;
//        $all_recover_image_buf
//        $all_recover_buf
//        $all_clear_image_buf
//        $all_clear_note_buf
        current_edit = 0;

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].show();
            note_fields[i].hide();
        }
        return false;
    }

    var PageState = {
        SELECT: 0,
        UPLOADING: 1
    };
    var total;
    var done = 0;
    var beginTime;
    var incr = function(){
        ++done;
    };
    var isAbort = false;
    var abort = function(){
        isAbort = true;
        updatePage(PageState.SELECT);
//        _isUploading = false;
        alert("stopped!");
    };
    var updatePage = function(mode){
        switch(mode){
            case PageState.SELECT:
                jQuery("#content-actions").show();
                jQuery("#uploadStatePnl").hide();
                jQuery("#display-options").show();
                break;
            case PageState.UPLOADING:
                jQuery("#display-options").hide();
                jQuery("#content-actions").hide();
                jQuery("#filePreviewPnl").hide();
                jQuery("#uploadStatePnl").show();
                updateProgress(0);
                break;
        }
    }
    var updateProgress = function(value){
        jQuery("#progressbar").progressbar("value", value);
    }
    var updateStatus = function(id){
        updateProgress(done / total * 100);

        var exec_file = id;
        if (done != total) {
            exec_file = id;
            jQuery("#progressmsg").text("$entry_id_message" + id + '$update_message' + " ( " + done + " / " + total + " $end_message )");
        } else {
            jQuery("#progressmsg").text(" $end_message(" + done + " / " + total + ")");
        }
    };

    var last_id = 0;
    var submit_all_edit = function() {
        var i = 0;
        var list = new Array();
        jQuery('#entry-table tr').each(function() {
            if (this.id) {
                list.push(this.id);
            }
        });
        var len = list.length;
        total = list.length;

        var retry = 0;
        var iter = function(){
            if (i < len - 1) {
                var id = list[i++];
                $all_submit_buf
                $asset_ids

//console.log(id + ":asset_ids_val"+asset_ids_val);

                var param = {};
                param['__mode'] = 'save_from_list';
                param['_type'] = 'entry';
                param['blog_id'] = blog_id;
                param['customfield_beacon'] = 1;
                $all_render_buf
                param['include_asset_ids'] = asset_ids_val; // overwrite note_include_asset_ids
                param['id'] = id;
                param['forward_list'] = 1;
                param['magic_token'] = 1;

                jQuery.ajax({
                    url: script_url,
                    type: "POST",
                    data: param,
//                    processData: false,
//                    contentType: false,
                    success: function() {
                        if (isAbort) {
                            return;
                        }
                        incr();
                        updateStatus(id);
                        retry = 0;
                        iter();
                    },
                    error: function(xhr, textStatus, errorThrown) {
                        if (isAbort) {
                            return;
                        }
                        console.log(xhr);
                        console.log(textStatus + ": " + errorThrown);

                        if (xhr.status == 500 && retry < 3) {
                            console.log("retry: " + this.id);

                            i--;
                            retry++;
                            iter();
                        } else {
                            alert( this.id + "is error");
                            if(confirm("continue?")){
                                iter();
                            } else {
                                abort();
                            }
                        }
                    }
                });
            } else if (i < len) {
                var id = list[i++];
                last_id = id;
                $all_submit_buf
                $asset_ids

                renderList({
                    __mode: 'save_from_list',
                    _type: 'entry',
                    blog_id: blog_id,
                    customfield_beacon: 1,
                    include_asset_ids: asset_ids_val,
                    $render_buf
                    id: id,
                    forward_list: 1,
                    magic_token: 1
                });
            } else {
                // complete
                incr();
                updateStatus(id);
            }
        };

        updateStatus(0);
//        beginTime = (new Date()).getTime();
//        _isUploading = true;

        jQuery('.indicator, #listing-table-overlay').show();
        jQuery('div.pagination').hide();
        jQuery('#msg-block div.msg-error.filtered-list-msg').remove()

        updatePage(PageState.UPLOADING);
        iter();
        cancelCreateMode();
        return false;
    };
HTML

    $node->innerHTML($html);
    $tmpl->insertBefore($node, $field);

#    my $plugin = MT->component("PowerListingFieldEditor");
#    my $node2 = $tmpl->createElement('setvarblock', { name => 'content_header', append => 1 });
#    my $html2 = 'Can use ' . rest_day() . 'days';
#    $node2->innerHTML($html2);
#    $tmpl->insertBefore($node2, $field);

    1;
}

sub add_css {
    my ($cb, $app, $template) = @_;
    return if is_limit();

    my $q = $app->param;
    my $mode = $q->param('__mode');
    return if !($mode eq 'list' || $mode eq 'filtered_list' || $mode eq 'save_from_list');

    my $type = $q->param('_type');
    return if !($type eq 'entry' || $type eq 'page' || $type eq 'comment' || $type eq 'asset' || $type eq 'member');

    my $blog_id = $app->param('blog_id') || 0;

    my $asset_type = [qw( file audio image video)] if $type eq 'asset';

    my @fields = CustomFields::Field->load({
        obj_type => ($asset_type) ? $asset_type : $type eq 'member' ? 'author' : $type,
        blog_id => $blog_id ? [ $blog_id, 0 ] : 0,
    });
#    return if !@fields;

    my $plugin = $app->component('PowerListingFieldEditor');
    my $text_width = $plugin->get_config_value('text_width', 'blog:'.$app->param('blog_id')) || 250;
    my $textarea_width = $plugin->get_config_value('textarea_width', 'blog:'.$app->param('blog_id')) || 400;
    my $text = $text_width - 15;
    my $textarea = $textarea_width - 20;

    my @text_selectors;
    my @textarea_selectors;
    my @button_selectors;
    my @button_selectors2;
    foreach my $field (@fields) {
        next if !( $field->type eq 'text' ||
                   $field->type eq 'textarea' ||
                   $field->type eq 'checkbox' ||
                   $field->type eq 'select' ||
                   $field->type eq 'radio' ||
                   $field->type eq 'url' ||
                   $field->type eq 'image' );
        if ( $field->type eq 'text' || $field->type eq 'url' ) {
            push @text_selectors, ".col.head.blog" . $blog_id . "_" . $field->basename;
        } elsif ( $field->type eq 'textarea' ) {
            push @textarea_selectors, ".col.head.blog" . $blog_id . "_" . $field->basename;
        } elsif ( $field->type eq 'checkbox' || $field->type eq 'radio' || $field->type eq 'select' || $field->type eq 'image' ) {
            push @button_selectors, ".col.head.blog" . $blog_id . "_" . $field->basename;
            if ( $field->type eq 'radio' ) {
            push @button_selectors2, "td.col.blog" . $blog_id . "_" . $field->basename . " .buttons";
            }
        }
    }

    my $title_selector;
    for my $col (qw( title text excerpt text_more keywords include_asset_ids tags )) {
        if ( $col =~ /text|excerpt/ ) {
            push @textarea_selectors, ".col.head.blog" . $blog_id . "_" . $col;
        } else {
            push @text_selectors, ".col.head.blog" . $blog_id . "_" . $col;
        }
        if ( $col eq 'title' ) {
            $title_selector = ".col.head.blog" . $blog_id . "_" . $col . '{ width: auto; min-width: 30px; }' .
                              ".col.blog" . $blog_id . "_" . $col . ' input { width: 100%; }';
        }
    }

    my $date_selector;
    for my $col (qw( authored_on modified_on unpublished_on )) {
        $date_selector .= ".col.head.blog" . $blog_id . "_" . $col . '{ /* max-width: 150px; width: 150px; */ }';
    }

    push @button_selectors, ".col.head.blog" . $blog_id . "_" . 'include_asset_ids';
    push @button_selectors, ".col.head." . 'category2';
    push @button_selectors, ".col.head." . 'status3';

#    my $text_selector = join(",", @text_selectors) . "{width: ${text_width}px;}" if @text_selectors;
    my $text_selector = join(",", @text_selectors) . "{width: 18%;}" if @text_selectors;
#    my $textarea_selector = join(",", @textarea_selectors) . "{width: ${textarea_width}px;}" if @textarea_selectors;
    my $textarea_selector = join(",", @textarea_selectors) . "{width: 30%;}" if @textarea_selectors;

#    my $button_selector = join(",", @button_selectors) . "{width: 140px;}" if @button_selectors;
    my $button_selector = join(",", @button_selectors) . "{width: 10%;}" if @button_selectors;
    my $button_selector2 = join(",", @button_selectors2) . "{display: block; margin-top: 3px;}" if @button_selectors2;

    my $old;
    my $new;

    $old = <<HTML;
<mt:var name="html_head">
HTML
    $old = quotemeta($old);
    $new = <<HTML;
<style type="text/css">
$date_selector
$text_selector
$textarea_selector
$button_selector
$button_selector2

$title_selector

.lfetext, .lfetextarea, .lfeurl, .lfedate {
    -moz-border-bottom-colors: none;
    -moz-border-image: none;
    -moz-border-left-colors: none;
    -moz-border-right-colors: none;
    -moz-border-top-colors: none;
    -moz-box-sizing: border-box;
    background-color: #FBFBFB;
    border-color: #ADADAD #DCDDDD #DCDDDD #ADADAD;
    border-radius: 3px 3px 3px 3px;
    border-right: 1px solid #DCDDDD;
    border-style: solid;
    border-width: 1px;
    box-shadow: 0 1px 0 #F3F3F3;
    font-size: 13px;
    outline: 0 none;
    padding: 0.2em 0.25em;
    vertical-align: middle;
    margin-bottom: 3px;
}
.lfetext, .lfeurl {
/*    width: ${text}px; */
    width: 100%;
}
.lfedate {
/*
    width: 135px;
    max-width: 135px;
*/
    width: 100%; 
}
.lfetextarea {
    vertical-align: bottom;
    height: 100px;
/*    width: ${textarea}px; */
    width: 100%; 
}
.lfecheckbox {
    margin-bottom: 3px;
}
.lfeselect {
    margin-bottom: 3px;
}
.lferadio .submit-note, .lferadio .cancel-note {
    margin-bottom: 3px;
}
.image-link {
    margin: 3px 0;
}
</style>
  <mt:var name="html_head">
HTML

    $$template =~ s/$old/$new/;

    $old = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>jqueryui/jquery-ui.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
HTML
    $old = quotemeta($old);
    $new = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>plugins/PowerListingFieldEditor/js/jquery-ui.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
<script>
        var script_url = '<mt:var name="script_url">';
        var blog_id = '<mt:var name="blog_id">';
        var magic_token = "<mt:var name="magic_token">";
</script>
HTML
    $$template =~ s/$old/$new/;

    $old = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>jqueryui/jquery-ui.min.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
HTML
    $old = quotemeta($old);
    $new = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>plugins/PowerListingFieldEditor/js/jquery-ui.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
<script>
        var script_url = '<mt:var name="script_url">';
        var blog_id = '<mt:var name="blog_id">';
        var magic_token = "<mt:var name="magic_token">";
</script>
HTML
    $$template =~ s/$old/$new/;
}

sub add_listing_data {
    my ($cb, $app, $template) = @_;
    my $old;
    my $new;

    $old = <<HTML;
</mt:setvarblock>
HTML
    $old = quotemeta($old);

    my $plugin = $app->component('PowerListingFieldEditor');
    my $display = $plugin->translate('Display');
    my $hidden = $plugin->translate('Hidden');

    $new = <<HTML;
</mt:setvarblock>
<mt:setvarblock name="content_header" append="1">
<link rel="stylesheet" type="text/css" href="<mt:var name="static_uri">plugins/PowerListingFieldEditor/css/jquery-ui-1.8.17.custom.css" />
<form class="create-inline" id="create-inline-notification" style="display: none;">
    <span class="buttons">
      <button
         type="submit"
         id="submit-all-field"
         class="button">
        <__trans phrase="Save">
      </button>
      <button
         type="button"
         class="button mt-cancel-action">
        <__trans phrase="Cancel">
      </button> 
    </span>
<span id="sidebar_none"><a href="javascript:void(0)">$hidden</a></span>
<span id="sidebar_block"><a href="javascript:void(0)" style="display:none">$display</a></span>
</form>
</mt:setvarblock>
<mt:setvarblock name="content_header" append="1">
<div id="uploadStatePnl">
    <p id="start_message" style="margin-bottom:5px"><mt:var name="uploading" /></p>
    <div id="progressbar"></div>
    <p id="progressmsg"></p>
</div>
</mt:setvarblock>
HTML
    $$template =~ s/$old/$new/;
}
########################################################

sub is_limit {
    return (rest_day() > 0) ? 0 : 1;
}

sub rest_day {
    my $plugin = MT->component("PowerListingFieldEditor");
    my $field_column = $plugin->get_config_value('field_column', 'system') || '';
    my $duration = time() - $field_column;

    my $rest_day = 31 - int($duration / (24 * 3600));

    return ($rest_day > 0) ? $rest_day : 0;
}

sub save_from_list {
    my $app = shift;

    return $app->forward('filtered_list') if is_limit();
#    $app->validate_magic or return;
    my $author = $app->user;
    my $type = $app->param('_type') || 'entry';
    $type = 'author' if $type eq 'member';

    my $class = $app->model($type)
        or return $app->errtrans("Invalid parameter");

    my $perms;
    if ( $type eq 'entry' || $type eq 'page' ) {
        $perms = $app->permissions
            or return $app->permission_denied();
    }

    my $id = $app->param('id');
    return if !$id;
    if ( $type eq 'page' ) {
        return $app->permission_denied()
            unless $perms->can_do('edit_all_pages');
    }

    require MT::Blog;
    my $blog_id = $app->param('blog_id');
    my $blog    = MT::Blog->load($blog_id)
        or return $app->error(
        $app->translate( 'Can\'t load blog #[_1].', $blog_id ) );
    my $obj = $class->load($id)
        || return $app->error(
        $app->translate( "No such [_1].", $class->class_label ) );

    if ($type ne 'author') {
        return $app->error( $app->translate("Invalid parameter") )
            unless $obj->blog_id == $blog_id;
    }

    if ( $type eq 'entry' ) {
        return $app->permission_denied()
            unless $perms->can_edit_entry( $obj, $author );
    }

    foreach ( $app->param() ) {
        next if $_ =~ /customfield_/;
        if ($_ eq 'title' || $_ eq 'text' || $_ eq 'text_more' || $_ eq 'excerpt' || $_ eq 'keywords' || $_ eq 'status') {
            $obj->$_($app->param($_));
        } elsif ($_ eq 'authored_on' || $_ eq 'modified_on' || $_ eq 'unpublished_on') {
            (my $ts = $app->param($_)) =~ s/(\d{4})-(\d{2})-(\d{2})\s(\d{2}):(\d{2}):(\d{2})/$1$2$3$4$5$6/;
            $obj->$_($ts) if $ts =~ /^\d{14}$/;
            if ($_ eq 'unpublished_on' and  !$app->param($_) ) {
                $obj->unpublished_on( undef ); 
            }
        } elsif ($_ eq 'tags') {
            my $tags = $app->param($_);
            if ( defined $tags ) {
                my $blog   = $app->blog;
                my $fields = $blog->smart_replace_fields;
                if ( $fields =~ m/tags/ig ) {
                    $tags = MT::App::CMS::_convert_word_chars( $app, $tags );
                }

                require MT::Tag;
                my $tag_delim = chr( $app->user->entry_prefs->{tag_delim} );
                my @tags = MT::Tag->split( $tag_delim, $tags );
                if (@tags) {
                    $obj->set_tags(@tags);
                }
                else {
                    $obj->remove_tags();
                }
            }
        } elsif ($_ eq 'category') {
            my $cat_id = $app->param($_);
            my $place  = MT::Placement->load(
                {   entry_id   => $id,
                    is_primary => 1
                }
            );
            if ( $place && !$cat_id ) {
                $place->remove
                    or return $app->error(
                    $app->translate(
                        "Removing placement failed: [_1]",
                        $place->errstr
                    )
                    );
            } elsif ($cat_id) {
                unless ($place) {
                    $place = MT::Placement->new;
                    $place->entry_id($id);
                    $place->blog_id($blog_id);
                    $place->is_primary(1);
                }
                $place->category_id($cat_id);
                $place->save
                    or return $app->error(
                    $app->translate(
                        "Saving placement failed: [_1]",
                        $place->errstr
                    )
                    );
            }
        }
    }
    $obj->modified_by( $author->id );
    $obj->save;

    CustomFields::App::CMS::CMSPostSave_customfield_objs('dummy', $app, $obj);

    # for asset
    require MT::Asset;
    require MT::ObjectAsset;
    my $include_asset_ids = $app->param('include_asset_ids');

    if ($include_asset_ids) {
        my @asset_ids;
        if ($include_asset_ids =~ /\[(.*)\]/) {
            @asset_ids = split( ',', $1 );
        } else {
            push @asset_ids, $include_asset_ids;
        }

        my $obj_assets        = ();
        my @obj_assets        = MT::ObjectAsset->load(
            { object_ds => 'entry', object_id => $obj->id } );
        foreach my $obj_asset (@obj_assets) {
            my $asset_id = $obj_asset->asset_id;
            $obj_assets->{$asset_id} = 1;
        }
        my $seen = ();
        foreach my $asset_id (@asset_ids) {
            my $obj_asset = MT::ObjectAsset->load(
                {   asset_id  => $asset_id,
                    object_ds => 'entry',
                    object_id => $obj->id
                }
            );
            unless ($obj_asset) {
                my $obj_asset = new MT::ObjectAsset;
                $obj_asset->blog_id($blog_id);
                $obj_asset->asset_id($asset_id);
                $obj_asset->object_ds('entry');
                $obj_asset->object_id( $obj->id );
                $obj_asset->save;
            }
            $seen->{$asset_id} = 1;
        }
        foreach my $asset_id ( keys %{$obj_assets} ) {
            unless ( $seen->{$asset_id} ) {
                my $obj_asset = MT::ObjectAsset->load(
                    {   asset_id  => $asset_id,
                        object_ds => 'entry',
                        object_id => $obj->id
                    }
                );
                $obj_asset->remove;
            }
        }
    } else {
        my @obj_assets = MT::ObjectAsset->load(
            { object_ds => 'entry', object_id => $obj->id } );
        return $app->forward('filtered_list') if !@obj_assets;
        foreach my $obj_asset ( @obj_assets ) {
            $obj_asset->remove;
        }
    }
    return $app->forward('filtered_list');
}

sub add_input_for_list {
    my $cb = shift;
    my ( $app, $res, $objs ) = @_;
    for my $data (@{$res->{objects}}) {
        my $id = $data->[0];
        my $entry = MT::Entry->load($id);
        next if !$entry;
        $data->[3] = format_ts( "%Y-%m-%d %k:%M:%S", $entry->authored_on, $entry->blog,
                $app->user ? $app->user->preferred_language : undef );
        $data->[4] = format_ts( "%Y-%m-%d %k:%M:%S", $entry->modified_on, $entry->blog,
                $app->user ? $app->user->preferred_language : undef );
#        my $author_id = $entry->author_id;
#        $data->[1] = '<input type="hidden" name="author_id_' . $id . '" value="' . $author_id . '" id="entry_author_id_' . $id . '">' . $data->[1];
    }
}

1;
